import io
import logging
import logging.handlers as logging_handlers
import queue
import tempfile

import picologging
import picologging.handlers as picologging_handlers


def filehandler_logging():
    with tempfile.NamedTemporaryFile() as f:
        logger = logging.Logger("test", logging.DEBUG)
        handler = logging.FileHandler(f.name)
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a logging issue")
        handler.close()


def filehandler_picologging():
    with tempfile.NamedTemporaryFile() as f:
        logger = picologging.Logger("test", picologging.DEBUG)
        handler = picologging.FileHandler(f.name)
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a picologging issue")
        handler.close()


def watchedfilehandler_logging():
    with tempfile.NamedTemporaryFile() as f:
        logger = logging.Logger("test", logging.DEBUG)
        handler = logging_handlers.WatchedFileHandler(f.name)
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a logging issue")
        handler.close()


def watchedfilehandler_picologging():
    with tempfile.NamedTemporaryFile() as f:
        logger = picologging.Logger("test", picologging.DEBUG)
        handler = picologging_handlers.WatchedFileHandler(f.name)
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a picologging issue")
        handler.close()


def rotatingfilehandler_logging():
    with tempfile.NamedTemporaryFile() as f:
        logger = logging.Logger("test", logging.DEBUG)
        handler = logging_handlers.RotatingFileHandler(
            f.name, maxBytes=10_000, backupCount=5
        )
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a logging issue")
        handler.close()


def rotatingfilehandler_picologging():
    with tempfile.NamedTemporaryFile() as f:
        logger = picologging.Logger("test", picologging.DEBUG)
        handler = picologging_handlers.RotatingFileHandler(
            f.name, maxBytes=10_000, backupCount=5
        )
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a picologging issue")
        handler.close()


def queuehandler_logging():
    logger = logging.Logger("test", picologging.DEBUG)
    q = queue.Queue()
    handler = logging_handlers.QueueHandler(q)
    logger.addHandler(handler)
    for _ in range(10_000):
        logger.debug("test")


def queuehandler_picologging():
    logger = picologging.Logger("test", picologging.DEBUG)
    q = queue.Queue()
    handler = picologging_handlers.QueueHandler(q)
    logger.addHandler(handler)
    for _ in range(10_000):
        logger.debug("test")


def queue_listener_logging():
    logger = logging.Logger("test", picologging.DEBUG)
    stream = io.StringIO()
    stream_handler = logging.StreamHandler(stream)
    q = queue.Queue()
    listener = logging_handlers.QueueListener(q, stream_handler)
    listener.start()
    handler = logging_handlers.QueueHandler(q)
    logger.addHandler(handler)
    for _ in range(1_000):
        logger.debug("test")

    listener.stop()


def queue_listener_picologging():
    logger = picologging.Logger("test", picologging.DEBUG)
    stream = io.StringIO()
    stream_handler = picologging.StreamHandler(stream)
    q = queue.Queue()
    listener = picologging_handlers.QueueListener(q, stream_handler)
    listener.start()
    handler = picologging_handlers.QueueHandler(q)
    logger.addHandler(handler)
    for _ in range(1_000):
        logger.debug("test")

    listener.stop()


def memoryhandler_logging():
    with tempfile.NamedTemporaryFile() as f:
        logger = logging.Logger("test", logging.DEBUG)
        target = logging.FileHandler(f.name)
        handler = logging_handlers.MemoryHandler(capacity=100, target=target)
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a logging issue")
        handler.close()


def memoryhandler_picologging():
    with tempfile.NamedTemporaryFile() as f:
        logger = picologging.Logger("test", picologging.DEBUG)
        target = picologging.FileHandler(f.name)
        handler = picologging_handlers.MemoryHandler(capacity=100, target=target)
        logger.handlers.append(handler)

        for _ in range(1_000):
            logger.debug("There has been a picologging issue")
        handler.close()


__benchmarks__ = [
    (filehandler_logging, filehandler_picologging, "FileHandler()"),
    (
        watchedfilehandler_logging,
        watchedfilehandler_picologging,
        "WatchedFileHandler()",
    ),
    (
        rotatingfilehandler_logging,
        rotatingfilehandler_picologging,
        "RotatingFileHandler()",
    ),
    (queuehandler_logging, queuehandler_picologging, "QueueHandler()"),
    (
        queue_listener_logging,
        queue_listener_picologging,
        "QueueListener() + QueueHandler()",
    ),
    (memoryhandler_logging, memoryhandler_picologging, "MemoryHandler()"),
]



import io
import queue
from typing import Any

import pytest

from picologging import INFO, Logger, StreamHandler, handlers

logger = Logger("test", INFO)
stream = io.StringIO()
stream_handler = StreamHandler(stream)
q = queue.Queue()
listener = handlers.QueueListener(q, stream_handler)
listener.start()
handler = handlers.QueueHandler(q)
logger.addHandler(handler)


@pytest.mark.xfail(reason="https://github.com/microsoft/picologging/issues/90")
def test_queue_listener_handler(caplog: Any) -> None:
    with caplog.at_level("INFO"):
        logger.info("Testing now!")
        assert "Testing now!" in caplog.text



import pytest

import picologging
from picologging.config import dictConfig, valid_ident


def test_dictconfig():
    class TestFilter(picologging.Filter):
        def __init__(self, param=None):
            self.param = param

        def filter(self, record):
            return True

    config = {
        "version": 1,
        "root": {"handlers": ["console"], "level": "DEBUG"},
        "loggers": {
            "test_config": {
                "handlers": ["console"],
                "level": "INFO",
                "propagate": True,
            },
        },
        "filters": {
            "test_filter": {"()": TestFilter},
            "example_filter": {},
        },
        "formatters": {
            "test_formatter": {"()": "picologging.Formatter"},
            "example_formatter": {"class": "picologging.Formatter"},
            "standard": {"format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"},
        },
        "handlers": {
            "console": {
                "class": "picologging.StreamHandler",
                "filters": ["test_filter"],
                "level": picologging.DEBUG,
            },
            "test": {
                "()": "picologging.StreamHandler",
                ".": {"level": picologging.DEBUG},
            },
            "console_formatted": {
                "class": "picologging.StreamHandler",
                "formatter": "standard",
            },
        },
    }

    dictConfig(config)

    root = picologging.getLogger()
    assert root.name == "root"
    assert root.level == picologging.DEBUG
    assert root.handlers[0].name == "console"
    assert isinstance(root.handlers[0], picologging.StreamHandler)

    logger = picologging.getLogger("test_config")
    assert logger.name == "test_config"
    assert logger.level == picologging.INFO
    assert logger.handlers[0].name == "console"
    assert isinstance(logger.handlers[0], picologging.StreamHandler)

    # Reset root logger
    picologging.root.handlers = []
    picologging.root.setLevel(picologging.WARNING)


def test_dictconfig_clear_existing_loggers(tmp_path):
    log_file = tmp_path / "log.txt"
    handler = picologging.FileHandler(log_file)
    logger = picologging.getLogger("test_config")
    logger.setLevel(picologging.DEBUG)
    logger.addHandler(handler)

    config = {
        "version": 1,
        "loggers": {
            "test_config": {
                "handlers": ["console"],
                "level": "INFO",
            },
        },
        "handlers": {
            "console": {
                "class": "picologging.StreamHandler",
            },
        },
    }

    dictConfig(config)

    logger = picologging.getLogger("test_config")
    assert logger.name == "test_config"
    assert logger.level == picologging.INFO
    assert len(logger.handlers) == 1
    assert logger.handlers[0].name == "console"
    assert isinstance(logger.handlers[0], picologging.StreamHandler)


def test_dictconfig_config_exceptions():
    with pytest.raises(ValueError):
        dictConfig({})

    with pytest.raises(ValueError):
        dictConfig({"version": 0})

    config = {
        "version": 1,
        "handlers": {
            "console": {
                "class": "picologging.SomeHandler",
            },
        },
    }

    with pytest.raises(ValueError):
        dictConfig(config)

    config = {
        "version": 1,
        "loggers": {
            "test_config": {"handlers": ["console"]},
        },
    }

    with pytest.raises(ValueError):
        dictConfig(config)


def test_config_exception_invalid_filter_for_handler():
    config = {
        "version": 1,
        "handlers": {
            "console": {
                "class": "picologging.StreamHandler",
                "filters": ["test_filter"],
                "level": picologging.DEBUG,
            },
        },
    }

    with pytest.raises(ValueError):
        dictConfig(config)


def test_dictconfig_incremental_not_supported():
    config = {"version": 1, "incremental": True}

    with pytest.raises(ValueError):
        dictConfig(config)


def test_dictconfig_formatters_exception():
    config = {
        "version": 1,
        "formatters": {
            "example_formatter": {"class": "picologging.NoFormatter"},
        },
    }

    with pytest.raises(ValueError):
        dictConfig(config)


def test_dictconfig_filters_exception():
    config = {
        "version": 1,
        "filters": {
            "example_filters": {"()": "picologging.NoFilter"},
        },
    }

    with pytest.raises(ValueError):
        dictConfig(config)


def test_reconfigure_dictconfig_with_child_loggers():
    logger = picologging.getLogger("test_config")
    logger.addHandler(picologging.StreamHandler())

    config = {
        "version": 1,
        "loggers": {
            "test_config.module": {
                "handlers": ["console"],
                "level": "INFO",
            },
        },
        "handlers": {
            "console": {
                "class": "picologging.StreamHandler",
            },
        },
    }

    dictConfig(config)
    logger = picologging.getLogger("test_config.module")
    assert logger.name == "test_config.module"
    assert logger.level == picologging.INFO
    assert len(logger.handlers) == 1
    assert logger.handlers[0].name == "console"
    assert isinstance(logger.handlers[0], picologging.StreamHandler)

    config = {
        "version": 1,
        "loggers": {
            "test_config": {
                "handlers": ["console"],
                "level": "INFO",
            },
        },
        "handlers": {
            "console": {
                "class": "picologging.StreamHandler",
            },
        },
    }

    dictConfig(config)
    logger = picologging.getLogger("test_config")
    assert logger.name == "test_config"
    assert logger.level == picologging.INFO
    assert len(logger.handlers) == 1
    assert logger.handlers[0].name == "console"
    assert isinstance(logger.handlers[0], picologging.StreamHandler)


def test_valid_ident():
    assert valid_ident("test")
    with pytest.raises(ValueError):
        valid_ident("test.test")
    with pytest.raises(ValueError):
        valid_ident("test test")
    with pytest.raises(ValueError):
        valid_ident("test-test")


def test_configure_with_filters():
    config = {
        "version": 1,
        "loggers": {
            "test_config": {
                "handlers": ["console"],
                "level": "INFO",
            },
        },
        "formatters": {
            "standard": {
                "format": "%(asctime)s %(levelname)s %(name)s::%(message)s",
                "validate": True,
            },
        },
        "handlers": {
            "console": {
                "class": "picologging.StreamHandler",
                "filters": ["test_filter"],
                "formatter": "standard",
            },
        },
        "filters": {
            "test_filter": {
                "()": "picologging.Filter",
                "name": "test_filter",
            },
        },
    }

    dictConfig(config)
    logger = picologging.getLogger("test_config")
    assert logger.name == "test_config"
    assert logger.level == picologging.INFO
    assert len(logger.handlers) == 1
    assert logger.handlers[0].name == "console"
    assert isinstance(logger.handlers[0], picologging.StreamHandler)
    assert len(logger.handlers[0].filters) == 1
    assert logger.handlers[0].filters[0].name == "test_filter"
    assert isinstance(logger.handlers[0].filters[0], picologging.Filter)
    assert (
        logger.handlers[0].formatter._fmt
        == "%(asctime)s %(levelname)s %(name)s::%(message)s"
    )


def test_configure_with_non_defined_handlers():
    config = {
        "version": 1,
        "loggers": {
            "test_config": {
                "handlers": ["potato"],
                "level": "INFO",
            },
        },
    }
    with pytest.raises(ValueError):
        dictConfig(config)


def test_config_existing_disabled_logger_90195():
    # See gh-90195
    config = {
        "version": 1,
        "disable_existing_loggers": False,
        "handlers": {
            "console": {
                "level": "DEBUG",
                "class": "logging.StreamHandler",
            },
        },
        "loggers": {"a": {"level": "DEBUG", "handlers": ["console"]}},
    }
    logger = picologging.getLogger("a")
    assert logger.disabled == False
    dictConfig(config)
    assert logger.disabled == False
    # Should disable all loggers ...
    dictConfig({"version": 1})
    assert logger.disabled == True
    del config["disable_existing_loggers"]
    dictConfig(config)
    # Logger should be enabled, since explicitly mentioned
    assert logger.disabled == False
